from strawberry.channels import GraphQLHTTPConsumer
from strawberry.channels.handlers.http_handler import ChannelsRequest
from strawberry.http.temporal_response import TemporalResponse
from kante.context import (
    ChannelsContext,
    EnhancendChannelsHTTPRequest,
)
from authentikate.utils import authenticate_header_or_none
from koherent.utils import get_assignation_id_or_none
from asgiref.sync import sync_to_async
import logging

logger = logging.getLogger(__name__)


class KanteHTTPConsumer(GraphQLHTTPConsumer):
    pass

    async def get_context(
        self, request: ChannelsRequest, response: TemporalResponse
    ) -> ChannelsContext:
        try:
            auth = await sync_to_async(authenticate_header_or_none)(request.headers)
            if auth:
                user = auth.user
                app = auth.app
                scopes = auth.token.scopes
                is_session = False

            else:
                user = request.consumer.scope.get("user", None)
                is_session = "session" in request.consumer.scope
                app = None
                scopes = None

            assignation_id = get_assignation_id_or_none(request.headers)

            return ChannelsContext(
                request=EnhancendChannelsHTTPRequest(
                    consumer=request.consumer,
                    body=request.body,
                    user=user,
                    app=app,
                    scopes=scopes,
                    is_session=is_session,
                    assignation_id=assignation_id,
                ),
                response=response,
            )
        except Exception as e:
            logger.error("Error in http get_context", exc_info=True)
            raise e



from typing import AsyncGenerator
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
from kante.types import Info
import logging


logger = logging.getLogger(__name__)


class Channel:
    """A GraphQL channel."""

    name = None

    def __init__(self, name) -> None:
        pass

    def broadcast(self, message, groups=None):
        """Broadcast a message to the given groups."""
        if groups is None:
            groups = ["default"]

        channel = get_channel_layer()

        for group in groups:
            logger.debug(f"Sending message to group {group}")
            async_to_sync(channel.group_send)(
                group,
                {
                    "type": f"channel.{self.name}",
                    "message": message,
                },
            )

    async def listen(self, info: Info, groups=None) -> AsyncGenerator[None, None]:
        if groups is None:
            groups = ["default"]
        ws = info.context.consumer
        channel_layer = ws.channel_layer

        for group in groups:
            # Join room group
            logger.debug(f"Joining group {group} for channel {ws.channel_name}")
            await channel_layer.group_add(group, ws.channel_name)

        async with ws.listen_to_channel(f"channel.{self.name}", groups=groups) as cm:
            async for message in cm:
                yield message["message"]


def build_channel(name):
    """Build a channel name for the given name."""

    x = Channel(name)
    return x.broadcast, x.listen



from strawberry.channels import ChannelsConsumer, ChannelsRequest
from strawberry.http.temporal_response import TemporalResponse
from dataclasses import dataclass
from typing import Any, Dict, Optional
from authentikate.models import App, User


@dataclass
class EnhancendChannelsHTTPRequest(ChannelsRequest):
    user: Optional[User] = None
    app: Optional[App] = None
    scopes: Optional[list[str]] = None
    assignation_id: Optional[str] = None
    is_session: bool = False

    def has_scopes(self, scopes: list[str]) -> bool:
        if self.is_session:
            return True

        if self.scopes is None:
            return False

        return all(scope in self.scopes for scope in scopes)


@dataclass
class ChannelsContext:
    request: EnhancendChannelsHTTPRequest
    response: TemporalResponse

    @property
    def session(self):
        # Depends on Channels' SessionMiddleware / AuthMiddlewareStack
        if "session" in self.request.consumer.scope:
            return self.request.consumer.scope["session"]

        return None


@dataclass
class EnhancendChannelsWSRequest:
    user: Optional[User] = None
    app: Optional[App] = None
    scopes: Optional[list[str]] = None
    assignation_id: Optional[str] = None


@dataclass
class ChannelsWSContext:
    request: EnhancendChannelsWSRequest
    consumer: ChannelsConsumer
    connection_params: Optional[Dict[str, Any]] = None

    @property
    def ws(self) -> ChannelsConsumer:
        return self.request
